@@ -3,7 +3,7 @@
 $block_name = _("Tasks Summary");
 
 /**
- * $Horde: nag/lib/Block/summary.php,v 1.51.8.9 2006/01/03 20:50:18 chuck Exp $
+ * $Horde: nag/lib/Block/summary.php,v 1.51.8.10 2006/06/06 00:05:00 chuck Exp $
  *
  * @package Horde_Block
  */
@@ -66,7 +66,7 @@ class Horde_Block_nag_summary extends Horde_Block {
         require_once dirname(__FILE__) . '/../base.php';
 
         $now = time();
-        $html = '<link href="' . htmlspecialchars(Horde::applicationUrl('themes/categoryCSS.php', true)) . '" rel="stylesheet" type="text/css" />';
+        $html = '';
 
         if (!empty($this->_params['show_alarms'])) {
             $messages = array();
@@ -184,9 +184,13 @@ class Horde_Block_nag_summary extends Horde_Block {
 
         if (empty($html)) {
             return '<em>' . _("No tasks to display") . '</em>';
-        } else {
-            return '<table cellspacing="0" width="100%" class="linedRow">' . $html . '</table>';
         }
+
+        return '<link href="' .
+            htmlspecialchars(Horde::applicationUrl('themes/categoryCSS.php', true)) .
+            '" rel="stylesheet" type="text/css" />' .
+            '<table cellspacing="0" width="100%" class="linedRow">' .
+            $html . '</table>';
     }
 
 }
