@@ -22,7 +22,7 @@
  *
  * The table structure can be created by the scripts/sql/nag.sql script.
  *
- * $Horde: nag/lib/Driver/sql.php,v 1.60.2.18 2006/08/08 17:16:04 jan Exp $
+ * $Horde: nag/lib/Driver/sql.php,v 1.60.2.19 2007/03/13 10:14:09 jan Exp $
  *
  * See the enclosed file COPYING for license information (GPL).  If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -397,8 +397,8 @@ class Nag_Driver_sql extends Nag_Driver {
         $q .= ' WHERE task_owner = ?';
         $q .= ' AND task_alarm > 0';
         $q .= ' AND (task_due - (task_alarm * 60) <= ?)';
-        $q .= ' AND task_due >= ?';
-        $values = array($this->_tasklist, $date, time());
+        $q .= ' AND task_completed = 0';
+        $values = array($this->_tasklist, $date);
 
         /* Log the query at a DEBUG log level. */
         Horde::logMessage(sprintf('SQL alarms list by %s: table = %s; query = "%s"',
