@@ -2,7 +2,7 @@
 /**
  * Nag external API interface.
  *
- * $Horde: nag/lib/api.php,v 1.100.10.12 2006/03/06 13:54:04 jan Exp $
+ * $Horde: nag/lib/api.php,v 1.100.10.16 2006/06/26 17:53:54 selsky Exp $
  *
  * This file defines Nag's external API interface. Other applications can
  * interact with Nag through this API.
@@ -12,7 +12,8 @@
 
 $_services['perms'] = array(
     'args' => array(),
-    'type' => '{urn:horde}hashHash');
+    'type' => '{urn:horde}hashHash'
+);
 
 $_services['show'] = array(
     'link' => '%application%/view.php?tasklist=|tasklist|&task=|task|&uid=|uid|',
@@ -20,7 +21,7 @@ $_services['show'] = array(
 
 $_services['browse'] = array(
     'args' => array('path' => 'string'),
-    'type' => '{urn:hash}hashHash',
+    'type' => '{urn:horde}hashHash',
 );
 
 $_services['addTasklist'] = array(
@@ -30,22 +31,22 @@ $_services['addTasklist'] = array(
 
 $_services['listTasklists'] = array(
     'args' => array('owneronly' => 'boolean', 'permission' => 'int'),
-    'type' => '{urn:hash}stringArray',
+    'type' => '{urn:horde}stringArray',
 );
 
 $_services['listTasks'] = array(
     'args' => array('sortby' => 'string', 'sortdir' => 'int'),
-    'type' => '{urn:hash}stringArray',
+    'type' => '{urn:horde}stringArray',
 );
 
 $_services['list'] = array(
     'args' => array(),
-    'type' => '{urn:hash}stringArray',
+    'type' => '{urn:horde}stringArray',
 );
 
 $_services['listBy'] = array(
     'args' => array('action' => 'string', 'timestamp' => 'int'),
-    'type' => '{urn:hash}stringArray',
+    'type' => '{urn:horde}stringArray',
 );
 
 $_services['getActionTimestamp'] = array(
@@ -59,12 +60,12 @@ $_services['import'] = array(
 );
 
 $_services['export'] = array(
-    'args' => array('uid' => 'string', 'contentType' => '{urn:hash}stringArray'),
+    'args' => array('uid' => 'string', 'contentType' => '{urn:horde}stringArray'),
     'type' => 'string',
 );
 
 $_services['delete'] = array(
-    'args' => array('uid' => '{urn:hash}stringArray'),
+    'args' => array('uid' => '{urn:horde}stringArray'),
     'type' => 'boolean',
 );
 
@@ -261,9 +262,12 @@ function _nag_listTasklists($owneronly, $permission)
  * Returns an array of UIDs for all tasks that the current user is authorized
  * to see.
  *
- * @return array  An array of UIDs for all tasks the user can access.
+ * @param variant $tasklist  The tasklist or an array of taskslists to list.
+ *
+ * @return array             An array of UIDs for all tasks
+ *                           the user can access.
  */
-function _nag_list()
+function _nag_list($tasklist = null)
 {
     require_once dirname(__FILE__) . '/base.php';
     global $conf;
@@ -273,7 +277,12 @@ function _nag_list()
         return PEAR::raiseError('Not configured');
     }
 
-    $tasks = Nag::listTasks();
+    if ($tasklist === null) {
+        $tasklist = Nag::getDefaultTasklist();
+    }
+
+    $tasks = Nag::listTasks(null,null,null,$tasklist);
+
     if (is_a($tasks, 'PEAR_Error')) {
         return $tasks;
     }
@@ -338,7 +347,10 @@ function _nag_getActionTimestamp($uid, $action, $tasklist = null)
 }
 
 /**
- * Imports a task represented in the specified content type.
+ * Imports one or more tasks represented in the specified content type.
+ *
+ * If a UID is present in the content and the task is already in the
+ * database, a replace is performed rather than an add.
  *
  * @param string $content      The content of the task.
  * @param string $contentType  What format is the data in? Currently supports:
@@ -349,11 +361,13 @@ function _nag_getActionTimestamp($uid, $action, $tasklist = null)
  *                             imported.  If 'null', the user's default
  *                             tasklist will be used.
  *
- * @return string  The new UID, or PEAR_Error on failure.
+ * @return string  The new UID on one import, an array of UIDs on multiple imports,
+ *                 or PEAR_Error on failure.
  */
 function _nag_import($content, $contentType, $tasklist = null)
 {
     require_once dirname(__FILE__) . '/base.php';
+
     global $prefs;
 
     if ($tasklist === null) {
@@ -364,8 +378,11 @@ function _nag_import($content, $contentType, $tasklist = null)
         return PEAR::raiseError(_("Permission Denied"));
     }
 
+    require_once 'Horde/iCalendar.php';
+
     /* Create a Nag_Driver instance. */
     require_once NAG_BASE . '/lib/Driver.php';
+
     $storage = &Nag_Driver::singleton($tasklist);
 
     switch ($contentType) {
@@ -373,120 +390,84 @@ function _nag_import($content, $contentType, $tasklist = null)
     case 'text/x-vcalendar':
     case 'text/x-vtodo':
     case 'text/calendar':
+        $iCal = &new Horde_iCalendar();
         if (!is_a($content, 'Horde_iCalendar_vtodo')) {
-            require_once 'Horde/iCalendar.php';
-            $iCal = &new Horde_iCalendar();
             if (!$iCal->parsevCalendar($content)) {
                 return PEAR::raiseError(_("There was an error importing the iCalendar data."));
             }
-
-            $components = $iCal->getComponents();
-            switch (count($components)) {
-            case 0:
-                return PEAR::raiseError(_("No iCalendar data was found."));
-
-            case 1:
-                $content = $components[0];
-                break;
-
-            default:
-                $ids = array();
-                foreach ($components as $content) {
-                    if (is_a($content, 'Horde_iCalendar_vtodo')) {
-                        $task = $storage->fromiCalendar($content);
-                        if (isset($task['uid']) && !is_a(($existing = $storage->getByUID($task['uid'])), 'PEAR_Error')) {
-                            $taskId = $existing['task_id'];
-                            $result = $storage->modify($taskId,
-                                isset($task['name']) ? $task['name'] : $existing['name'],
-                                isset($task['desc']) ? $task['desc'] : $existing['desc'],
-                                isset($task['due']) ? $task['due'] : $existing['due'],
-                                isset($task['priority']) ? $task['priority'] : $existing['priority'],
-                                isset($task['completed']) ? (int)$task['completed'] : $existing['completed'],
-                                isset($task['category']) ? $task['category'] : $existing['category'],
-                                isset($task['alarm']) ? $task['alarm'] : $existing['alarm']);
-                            if (is_a($result, 'PEAR_Error')) {
-                                return $result;
-                            }
-                        } else {
-                            $taskId = $storage->add(
-                                isset($task['name']) ? $task['name'] : '',
-                                isset($task['desc']) ? $task['desc'] : '',
-                                isset($task['due']) ? $task['due'] : 0,
-                                isset($task['priority']) ? $task['priority'] : 3,
-                                !empty($task['completed']) ? 1 : 0,
-                                isset($task['category']) ? $task['category'] : '',
-                                isset($task['alarm']) ? $task['alarm'] : 0,
-                                isset($task['uid']) ? $task['uid'] : null);
-                            if (is_a($taskId, 'PEAR_Error')) {
-                                return $taskId;
-                            }
-                        }
-                        $ids[] = $taskId;
-                    }
-                }
-                return $ids;
-            }
-        }
-
-        $task = $storage->fromiCalendar($content);
-
-        if (isset($task['uid']) && !is_a(($existing = $storage->getByUID($task['uid'])), 'PEAR_Error')) {
-            $taskId = $existing['task_id'];
-            $result = $storage->modify($taskId,
-                                       isset($task['name']) ? $task['name'] : $existing['name'],
-                                       isset($task['desc']) ? $task['desc'] : $existing['desc'],
-                                       isset($task['due']) ? $task['due'] : $existing['due'],
-                                       isset($task['priority']) ? $task['priority'] : $existing['priority'],
-                                       isset($task['completed']) ? (int)$task['completed'] : $existing['completed'],
-                                       isset($task['category']) ? $task['category'] : $existing['category'],
-                                       isset($task['alarm']) ? $task['alarm'] : $existing['alarm']);
-            if (is_a($result, 'PEAR_Error')) {
-                return $result;
-            }
         } else {
-            $taskId = $storage->add(isset($task['name']) ? $task['name'] : '',
-                                    isset($task['desc']) ? $task['desc'] : '',
-                                    isset($task['due']) ? $task['due'] : 0,
-                                    isset($task['priority']) ? $task['priority'] : 3,
-                                    !empty($task['completed']) ? 1 : 0,
-                                    isset($task['category']) ? $task['category'] : '',
-                                    isset($task['alarm']) ? $task['alarm'] : 0,
-                                    isset($task['uid']) ? $task['uid'] : null);
+            $iCal->addComponent($content);
         }
-        break;
 
-    default:
-        return PEAR::raiseError(sprintf(_("Unsupported Content-Type: %s"), $contentType));
+        $components = $iCal->getComponents();
+        if (count($components) == 0) {
+            return PEAR::raiseError(_("No iCalendar data was found."));
+        }
 
+        $ids = array();
+        foreach ($components as $content) {
+            if (is_a($content, 'Horde_iCalendar_vtodo')) {
+                $task = $storage->fromiCalendar($content);
+                if (isset($task['uid']) && !is_a(($existing = $storage->getByUID($task['uid'])), 'PEAR_Error')) {
+                    $taskId = $existing['task_id'];
+                    $result = $storage->modify($taskId,
+                        isset($task['name']) ? $task['name'] : $existing['name'],
+                        isset($task['desc']) ? $task['desc'] : $existing['desc'],
+                        isset($task['due']) ? $task['due'] : $existing['due'],
+                        isset($task['priority']) ? $task['priority'] : $existing['priority'],
+                        isset($task['completed']) ? (int)$task['completed'] : $existing['completed'],
+                        isset($task['category']) ? $task['category'] : $existing['category'],
+                        isset($task['alarm']) ? $task['alarm'] : $existing['alarm']
+                        );
+
+                    if (is_a($result, 'PEAR_Error')) {
+                        return $result;
+                    }
+                    $ids[] = $task['uid'];
+                } else {
+                    $newTask = $storage->add(
+                        isset($task['name']) ? $task['name'] : '',
+                        isset($task['desc']) ? $task['desc'] : '',
+                        isset($task['due']) ? $task['due'] : 0,
+                        isset($task['priority']) ? $task['priority'] : 3,
+                        !empty($task['completed']) ? 1 : 0,
+                        isset($task['category']) ? $task['category'] : '',
+                        isset($task['alarm']) ? $task['alarm'] : 0,
+                        isset($task['uid']) ? $task['uid'] : null
+                        );
+
+                    if (is_a($newTask, 'PEAR_Error')) {
+                        return $newTask;
+                    }
+                    $ids[] = $newTask[1]; // use UID rather than ID
+                }
+
+            }
+        }
+        if (count($ids) == 0) {
+            return PEAR::raiseError(_("No iCalendar data was found."));
+        } else if (count($ids) == 1) {
+            return $ids[0];
+        }
+        return $ids;
     }
 
-    if (is_a($taskId, 'PEAR_Error')) {
-        return $taskId;
-    }
+    return PEAR::raiseError(sprintf(_("Unsupported Content-Type: %s"), $contentType));
 
-    $task = $storage->get($taskId);
-    return $task['uid'];
 }
 
 /**
  * Exports a task, identified by UID, in the requested content type.
  *
  * @param string $uid         Identify the task to export.
- * @param mixed $contentType  What format should the data be in?
- *                            Either a string with one of:
+ * @param string $contentType  What format should the data be in?
+ *                            A string with one of:
  *                            <pre>
- *                             text/x-vtodo
- *                             text/x-vcalendar
+ *                             text/calendar (VCALENDAR 2.0. Recommended as this is specified in rfc2445)
+ *                             text/x-vtodo (seems to be used by horde only. Do we need this?)
+ *                             text/x-vcalendar (old VCALENDAR 1.0 format. Still in wide use)
  *                             text/x-icalendar
  *                            </pre>
- *                            or an array with options:
- *                            <pre>
- *                             'ContentType':  as above
- *                             'ENCODING': (optional) character encoding
- *                                         for strings fields
- *                             'CHARSET':  (optional) charset. Like UTF-8
- *                             'VERSION':  (optional) Format version (ex. iCalendar 2.0, vCalendar 1.0).
- *                            </pre>
  *
  * @return string  The requested data.
  */
@@ -505,23 +486,10 @@ function _nag_export($uid, $contentType)
     }
 
     $version = '2.0';
-    if (is_array($contentType)) {
-        $options = $contentType;
-        $contentType = $options['ContentType'];
-        unset($options['ContentType']);
-        if (isset($options['VERSION'])) {
-            $version = $options['VERSION'];
-            unset($options['VERSION']);
-        }
-    } else {
-        $options = array();
-    }
-
     switch ($contentType) {
     case 'text/x-vcalendar':
-        // Downgrade version and fall through.
         $version = '1.0';
-
+    case 'text/calendar':
     case 'text/x-icalendar':
     case 'text/x-vtodo':
         require_once dirname(__FILE__) . '/version.php';
@@ -529,16 +497,12 @@ function _nag_export($uid, $contentType)
 
         // Create the new iCalendar container.
         $iCal = &new Horde_iCalendar($version);
-        $iCal->setAttribute('VERSION', $version);
         $iCal->setAttribute('PRODID', '-//The Horde Project//Nag ' . NAG_VERSION . '//EN');
         $iCal->setAttribute('METHOD', 'PUBLISH');
 
         // Create new vTodo object.
         $vTodo = $storage->toiCalendar($task, $iCal);
-
-        // Set encoding options for all string values.
-        $vTodo->setParameter('SUMMARY', $options);
-        $vTodo->setParameter('DESCRIPTION', $options);
+        $vTodo->setAttribute('VERSION', $version);
 
         $iCal->addComponent($vTodo);
 
@@ -591,12 +555,18 @@ function _nag_delete($uid)
  * Replaces the task identified by UID with the content represented in the
  * specified content type.
  *
+ * If you want to replace multiple tasks with the UID specified in the
+ * VCALENDAR data, you may use _nag_import instead. This automatically
+ * does a replace if existings UIDs are found.
+ *
+ *
  * @param string $uid          Identify the task to replace.
  * @param string $content      The content of the task.
  * @param string $contentType  What format is the data in? Currently supports:
  *                             text/x-icalendar
  *                             text/x-vcalendar
  *                             text/x-vtodo
+ *                             text/calendar
  *
  * @return boolean  Success or failure.
  */
@@ -605,13 +575,13 @@ function _nag_replace($uid, $content, $contentType)
     require_once dirname(__FILE__) . '/base.php';
 
     $storage = &Nag_Driver::singleton();
-    $task = $storage->getByUID($uid);
-    if (is_a($task, 'PEAR_Error')) {
-        return $task;
+    $existing = $storage->getByUID($uid);
+    if (is_a($existing, 'PEAR_Error')) {
+        return $existing;
     }
-    $taskId = $task['task_id'];
+    $taskId = $existing['task_id'];
 
-    if (!array_key_exists($task['tasklist_id'], Nag::listTasklists(false, PERMS_EDIT))) {
+    if (!array_key_exists($existing['tasklist_id'], Nag::listTasklists(false, PERMS_EDIT))) {
         return PEAR::raiseError(_("Permission Denied"));
     }
 
@@ -643,14 +613,15 @@ function _nag_replace($uid, $content, $contentType)
 
         $task = $storage->fromiCalendar($content);
         $result = $storage->modify($taskId,
-                                   isset($task['name']) ? $task['name'] : '',
-                                   isset($task['desc']) ? $task['desc'] : '',
-                                   isset($task['due']) ? $task['due'] : '',
-                                   isset($task['priority']) ? $task['priority'] : 3,
-                                   isset($task['completed']) ? (int)$task['completed'] : 0,
-                                   isset($task['category']) ? $task['category'] : '',
-                                   isset($task['alarm']) ? $task['alarm'] : 0
-                                   );
+                                    isset($task['name']) ? $task['name'] : $existing['name'],
+                                    isset($task['desc']) ? $task['desc'] : $existing['desc'],
+                                    isset($task['due']) ? $task['due'] : $existing['due'],
+                                    isset($task['priority']) ? $task['priority'] : $existing['priority'],
+                                    isset($task['completed']) ? (int)$task['completed'] : $existing['completed'],
+                                    isset($task['category']) ? $task['category'] : $existing['category'],
+                                    isset($task['alarm']) ? $task['alarm'] : $existing['alarm']
+                                    );
+
         break;
 
     default:
